@@ -1,7 +1,4 @@
 # flake8: noqa
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
 
 from ._version import get_versions
 __version__ = get_versions()['version']
