@@ -1,21 +1,16 @@
 """String formatting routines for __repr__.
-
-For the sake of sanity, we only do internal formatting with unicode, which can
-be returned by the __unicode__ special method. We use ReprMixin to provide the
-__repr__ method so that things can work on Python 2.
 """
-from __future__ import absolute_import, division, print_function
-
 import contextlib
 import functools
 from datetime import datetime, timedelta
+from itertools import zip_longest
 
 import numpy as np
 import pandas as pd
 
+from .duck_array_ops import array_equiv
 from .options import OPTIONS
-from .pycompat import (
-    PY2, bytes_type, dask_array_type, unicode_type, zip_longest)
+from .pycompat import dask_array_type
 
 try:
     from pandas.errors import OutOfBoundsDatetime
@@ -34,9 +29,9 @@ def pretty_print(x, numchars):
 
 
 def maybe_truncate(obj, maxlen=500):
-    s = unicode_type(obj)
+    s = str(obj)
     if len(s) > maxlen:
-        s = s[:(maxlen - 3)] + u'...'
+        s = s[:(maxlen - 3)] + '...'
     return s
 
 
@@ -47,24 +42,6 @@ def wrap_indent(text, start='', length=None):
     return start + indent.join(x for x in text.splitlines())
 
 
-def ensure_valid_repr(string):
-    """Ensure that the given value is valid for the result of __repr__.
-
-    On Python 2, this means we need to convert unicode to bytes. We won't need
-    this function once we drop Python 2.7 support.
-    """
-    if PY2 and isinstance(string, unicode_type):
-        string = string.encode('utf-8')
-    return string
-
-
-class ReprMixin(object):
-    """Mixin that defines __repr__ for a class that already has __unicode__."""
-
-    def __repr__(self):
-        return ensure_valid_repr(self.__unicode__())
-
-
 def _get_indexer_at_least_n_items(shape, n_desired, from_end):
     assert 0 < n_desired <= np.prod(shape)
     cum_items = np.cumprod(shape[::-1])
@@ -126,9 +103,9 @@ def format_timestamp(t):
     """Cast given object to a Timestamp and return a nicely formatted string"""
     # Timestamp is only valid for 1678 to 2262
     try:
-        datetime_str = unicode_type(pd.Timestamp(t))
+        datetime_str = str(pd.Timestamp(t))
     except OutOfBoundsDatetime:
-        datetime_str = unicode_type(t)
+        datetime_str = str(t)
 
     try:
         date_str, time_str = datetime_str.split()
@@ -144,7 +121,7 @@ def format_timestamp(t):
 
 def format_timedelta(t, timedelta_format=None):
     """Cast given object to a Timestamp and return a nicely formatted string"""
-    timedelta_str = unicode_type(pd.Timedelta(t))
+    timedelta_str = str(pd.Timedelta(t))
     try:
         days_str, time_str = timedelta_str.split(' days ')
     except ValueError:
@@ -165,12 +142,12 @@ def format_item(x, timedelta_format=None, quote_strings=True):
         return format_timestamp(x)
     if isinstance(x, (np.timedelta64, timedelta)):
         return format_timedelta(x, timedelta_format=timedelta_format)
-    elif isinstance(x, (unicode_type, bytes_type)):
+    elif isinstance(x, (str, bytes)):
         return repr(x) if quote_strings else x
     elif isinstance(x, (float, np.float)):
-        return u'{0:.4}'.format(x)
+        return '{0:.4}'.format(x)
     else:
-        return unicode_type(x)
+        return str(x)
 
 
 def format_items(x):
@@ -214,20 +191,20 @@ def format_array_flat(array, max_width):
     cum_len = np.cumsum([len(s) + 1 for s in relevant_items]) - 1
     if (array.size > 2) and ((max_possibly_relevant < array.size) or
                              (cum_len > max_width).any()):
-        padding = u' ... '
+        padding = ' ... '
         count = min(array.size,
                     max(np.argmax(cum_len + len(padding) - 1 > max_width), 2))
     else:
         count = array.size
-        padding = u'' if (count <= 1) else u' '
+        padding = '' if (count <= 1) else ' '
 
     num_front = (count + 1) // 2
     num_back = count - num_front
     # note that num_back is 0 <--> array.size is 0 or 1
     #                         <--> relevant_back_items is []
-    pprint_str = (u' '.join(relevant_front_items[:num_front]) +
+    pprint_str = (' '.join(relevant_front_items[:num_front]) +
                   padding +
-                  u' '.join(relevant_back_items[-num_back:]))
+                  ' '.join(relevant_back_items[-num_back:]))
     return pprint_str
 
 
@@ -235,30 +212,30 @@ def summarize_variable(name, var, col_width, show_values=True,
                        marker=' ', max_width=None):
     if max_width is None:
         max_width = OPTIONS['display_width']
-    first_col = pretty_print(u'  %s %s ' % (marker, name), col_width)
+    first_col = pretty_print('  %s %s ' % (marker, name), col_width)
     if var.dims:
-        dims_str = u'(%s) ' % u', '.join(map(unicode_type, var.dims))
+        dims_str = '(%s) ' % ', '.join(map(str, var.dims))
     else:
-        dims_str = u''
-    front_str = u'%s%s%s ' % (first_col, dims_str, var.dtype)
+        dims_str = ''
+    front_str = '%s%s%s ' % (first_col, dims_str, var.dtype)
     if show_values:
         values_str = format_array_flat(var, max_width - len(front_str))
     elif isinstance(var._data, dask_array_type):
         values_str = short_dask_repr(var, show_dtype=False)
     else:
-        values_str = u'...'
+        values_str = '...'
 
     return front_str + values_str
 
 
 def _summarize_coord_multiindex(coord, col_width, marker):
-    first_col = pretty_print(u'  %s %s ' % (marker, coord.name), col_width)
-    return u'%s(%s) MultiIndex' % (first_col, unicode_type(coord.dims[0]))
+    first_col = pretty_print('  %s %s ' % (marker, coord.name), col_width)
+    return '%s(%s) MultiIndex' % (first_col, str(coord.dims[0]))
 
 
-def _summarize_coord_levels(coord, col_width, marker=u'-'):
+def _summarize_coord_levels(coord, col_width, marker='-'):
     relevant_coord = coord[:30]
-    return u'\n'.join(
+    return '\n'.join(
         [summarize_variable(lname,
                             relevant_coord.get_level_variable(lname),
                             col_width, marker=marker)
@@ -273,11 +250,11 @@ def summarize_datavar(name, var, col_width):
 def summarize_coord(name, var, col_width):
     is_index = name in var.dims
     show_values = var._in_memory
-    marker = u'*' if is_index else u' '
+    marker = '*' if is_index else ' '
     if is_index:
         coord = var.variable.to_index_variable()
         if coord.level_names is not None:
-            return u'\n'.join(
+            return '\n'.join(
                 [_summarize_coord_multiindex(coord, col_width, marker),
                  _summarize_coord_levels(coord, col_width)])
     return summarize_variable(
@@ -287,16 +264,16 @@ def summarize_coord(name, var, col_width):
 def summarize_attr(key, value, col_width=None):
     """Summary for __repr__ - use ``X.attrs[key]`` for full value."""
     # Indent key and add ':', then right-pad if col_width is not None
-    k_str = u'    %s:' % key
+    k_str = '    %s:' % key
     if col_width is not None:
         k_str = pretty_print(k_str, col_width)
     # Replace tabs and newlines, so we print on one line in known width
-    v_str = unicode_type(value).replace(u'\t', u'\\t').replace(u'\n', u'\\n')
+    v_str = str(value).replace('\t', '\\t').replace('\n', '\\n')
     # Finally, truncate to the desired display width
-    return maybe_truncate(u'%s %s' % (k_str, v_str), OPTIONS['display_width'])
+    return maybe_truncate('%s %s' % (k_str, v_str), OPTIONS['display_width'])
 
 
-EMPTY_REPR = u'    *empty*'
+EMPTY_REPR = '    *empty*'
 
 
 def _get_col_items(mapping):
@@ -317,7 +294,7 @@ def _get_col_items(mapping):
 
 
 def _calculate_col_width(col_items):
-    max_name_length = (max(len(unicode_type(s)) for s in col_items)
+    max_name_length = (max(len(str(s)) for s in col_items)
                        if col_items else 0)
     col_width = max(max_name_length, 7) + 6
     return col_width
@@ -326,26 +303,26 @@ def _calculate_col_width(col_items):
 def _mapping_repr(mapping, title, summarizer, col_width=None):
     if col_width is None:
         col_width = _calculate_col_width(mapping)
-    summary = [u'%s:' % title]
+    summary = ['%s:' % title]
     if mapping:
         summary += [summarizer(k, v, col_width) for k, v in mapping.items()]
     else:
         summary += [EMPTY_REPR]
-    return u'\n'.join(summary)
+    return '\n'.join(summary)
 
 
-data_vars_repr = functools.partial(_mapping_repr, title=u'Data variables',
+data_vars_repr = functools.partial(_mapping_repr, title='Data variables',
                                    summarizer=summarize_datavar)
 
 
-attrs_repr = functools.partial(_mapping_repr, title=u'Attributes',
+attrs_repr = functools.partial(_mapping_repr, title='Attributes',
                                summarizer=summarize_attr)
 
 
 def coords_repr(coords, col_width=None):
     if col_width is None:
         col_width = _calculate_col_width(_get_col_items(coords))
-    return _mapping_repr(coords, title=u'Coordinates',
+    return _mapping_repr(coords, title='Coordinates',
                          summarizer=summarize_coord, col_width=col_width)
 
 
@@ -353,19 +330,19 @@ def indexes_repr(indexes):
     summary = []
     for k, v in indexes.items():
         summary.append(wrap_indent(repr(v), '%s: ' % k))
-    return u'\n'.join(summary)
+    return '\n'.join(summary)
 
 
 def dim_summary(obj):
-    elements = [u'%s: %s' % (k, v) for k, v in obj.sizes.items()]
-    return u', '.join(elements)
+    elements = ['%s: %s' % (k, v) for k, v in obj.sizes.items()]
+    return ', '.join(elements)
 
 
 def unindexed_dims_repr(dims, coords):
     unindexed_dims = [d for d in dims if d not in coords]
     if unindexed_dims:
-        dims_str = u', '.join(u'%s' % d for d in unindexed_dims)
-        return u'Dimensions without coordinates: ' + dims_str
+        dims_str = ', '.join('%s' % d for d in unindexed_dims)
+        return 'Dimensions without coordinates: ' + dims_str
     else:
         return None
 
@@ -411,22 +388,26 @@ def short_dask_repr(array, show_dtype=True):
         return 'dask.array<shape=%s, chunksize=%s>' % (array.shape, chunksize)
 
 
+def short_data_repr(array):
+    if isinstance(getattr(array, 'variable', array)._data, dask_array_type):
+        return short_dask_repr(array)
+    elif array._in_memory or array.size < 1e5:
+        return short_array_repr(array.values)
+    else:
+        return u'[%s values with dtype=%s]' % (array.size, array.dtype)
+
+
 def array_repr(arr):
     # used for DataArray, Variable and IndexVariable
     if hasattr(arr, 'name') and arr.name is not None:
         name_str = '%r ' % arr.name
     else:
-        name_str = u''
+        name_str = ''
 
-    summary = [u'<xarray.%s %s(%s)>'
+    summary = ['<xarray.%s %s(%s)>'
                % (type(arr).__name__, name_str, dim_summary(arr))]
 
-    if isinstance(getattr(arr, 'variable', arr)._data, dask_array_type):
-        summary.append(short_dask_repr(arr))
-    elif arr._in_memory or arr.size < 1e5:
-        summary.append(short_array_repr(arr.values))
-    else:
-        summary.append(u'[%s values with dtype=%s]' % (arr.size, arr.dtype))
+    summary.append(short_data_repr(arr))
 
     if hasattr(arr, 'coords'):
         if arr.coords:
@@ -439,16 +420,16 @@ def array_repr(arr):
     if arr.attrs:
         summary.append(attrs_repr(arr.attrs))
 
-    return u'\n'.join(summary)
+    return '\n'.join(summary)
 
 
 def dataset_repr(ds):
-    summary = [u'<xarray.%s>' % type(ds).__name__]
+    summary = ['<xarray.%s>' % type(ds).__name__]
 
     col_width = _calculate_col_width(_get_col_items(ds.variables))
 
-    dims_start = pretty_print(u'Dimensions:', col_width)
-    summary.append(u'%s(%s)' % (dims_start, dim_summary(ds)))
+    dims_start = pretty_print('Dimensions:', col_width)
+    summary.append('%s(%s)' % (dims_start, dim_summary(ds)))
 
     if ds.coords:
         summary.append(coords_repr(ds.coords, col_width=col_width))
@@ -462,4 +443,133 @@ def dataset_repr(ds):
     if ds.attrs:
         summary.append(attrs_repr(ds.attrs))
 
-    return u'\n'.join(summary)
+    return '\n'.join(summary)
+
+
+def diff_dim_summary(a, b):
+    if a.dims != b.dims:
+        return "Differing dimensions:\n    ({}) != ({})".format(
+            dim_summary(a), dim_summary(b))
+    else:
+        return ""
+
+
+def _diff_mapping_repr(a_mapping, b_mapping, compat,
+                       title, summarizer, col_width=None):
+
+    def extra_items_repr(extra_keys, mapping, ab_side):
+        extra_repr = [summarizer(k, mapping[k], col_width) for k in extra_keys]
+        if extra_repr:
+            header = "{} only on the {} object:".format(title, ab_side)
+            return [header] + extra_repr
+        else:
+            return []
+
+    a_keys = set(a_mapping)
+    b_keys = set(b_mapping)
+
+    summary = []
+
+    diff_items = []
+
+    for k in a_keys & b_keys:
+        try:
+            # compare xarray variable
+            compatible = getattr(a_mapping[k], compat)(b_mapping[k])
+            is_variable = True
+        except AttributeError:
+            # compare attribute value
+            compatible = a_mapping[k] == b_mapping[k]
+            is_variable = False
+
+        if not compatible:
+            temp = [summarizer(k, vars[k], col_width)
+                    for vars in (a_mapping, b_mapping)]
+
+            if compat == 'identical' and is_variable:
+                attrs_summary = []
+
+                for m in (a_mapping, b_mapping):
+                    attr_s = "\n".join([summarize_attr(ak, av)
+                                        for ak, av in m[k].attrs.items()])
+                    attrs_summary.append(attr_s)
+
+                temp = ["\n".join([var_s, attr_s]) if attr_s else var_s
+                        for var_s, attr_s in zip(temp, attrs_summary)]
+
+            diff_items += [ab_side + s[1:]
+                           for ab_side, s in zip(('L', 'R'), temp)]
+
+    if diff_items:
+        summary += ["Differing {}:".format(title.lower())] + diff_items
+
+    summary += extra_items_repr(a_keys - b_keys, a_mapping, "left")
+    summary += extra_items_repr(b_keys - a_keys, b_mapping, "right")
+
+    return "\n".join(summary)
+
+
+diff_coords_repr = functools.partial(_diff_mapping_repr,
+                                     title="Coordinates",
+                                     summarizer=summarize_coord)
+
+
+diff_data_vars_repr = functools.partial(_diff_mapping_repr,
+                                        title="Data variables",
+                                        summarizer=summarize_datavar)
+
+
+diff_attrs_repr = functools.partial(_diff_mapping_repr,
+                                    title="Attributes",
+                                    summarizer=summarize_attr)
+
+
+def _compat_to_str(compat):
+    if compat == "equals":
+        return "equal"
+    else:
+        return compat
+
+
+def diff_array_repr(a, b, compat):
+    # used for DataArray, Variable and IndexVariable
+    summary = ["Left and right {} objects are not {}"
+               .format(type(a).__name__, _compat_to_str(compat))]
+
+    summary.append(diff_dim_summary(a, b))
+
+    if not array_equiv(a.data, b.data):
+        temp = [wrap_indent(short_array_repr(obj), start='    ')
+                for obj in (a, b)]
+        diff_data_repr = [ab_side + "\n" + ab_data_repr
+                          for ab_side, ab_data_repr in zip(('L', 'R'), temp)]
+        summary += ["Differing values:"] + diff_data_repr
+
+    if hasattr(a, 'coords'):
+        col_width = _calculate_col_width(set(a.coords) | set(b.coords))
+        summary.append(diff_coords_repr(a.coords, b.coords, compat,
+                                        col_width=col_width))
+
+    if compat == 'identical':
+        summary.append(diff_attrs_repr(a.attrs, b.attrs, compat))
+
+    return "\n".join(summary)
+
+
+def diff_dataset_repr(a, b, compat):
+    summary = ["Left and right {} objects are not {}"
+               .format(type(a).__name__, _compat_to_str(compat))]
+
+    col_width = _calculate_col_width(
+        set(_get_col_items(a.variables) + _get_col_items(b.variables)))
+
+    summary.append(diff_dim_summary(a, b))
+    summary.append(diff_coords_repr(a.coords, b.coords, compat,
+                                    col_width=col_width))
+    summary.append(diff_data_vars_repr(a.data_vars, b.data_vars, compat,
+                                       col_width=col_width))
+
+    if compat == 'identical':
+        summary.append(diff_attrs_repr(a.attrs, b.attrs, compat))
+
+    return "\n".join(summary)
