@@ -1,25 +1,23 @@
-from __future__ import absolute_import, division, print_function
-
 import warnings
-from collections import Iterable
+from collections.abc import Iterable
 from functools import partial
+from typing import Any, Dict
 
 import numpy as np
 import pandas as pd
 
-from . import rolling
+from . import utils
 from .common import _contains_datetime_like_objects
 from .computation import apply_ufunc
-from .duck_array_ops import dask_array_type
-from .pycompat import iteritems
-from .utils import OrderedSet, datetime_to_numeric, is_scalar
+from .duck_array_ops import dask_array_type, datetime_to_numeric
+from .utils import OrderedSet, is_scalar
 from .variable import Variable, broadcast_variables
 
 
 class BaseInterpolator(object):
     '''gerneric interpolator class for normalizing interpolation methods'''
-    cons_kwargs = {}
-    call_kwargs = {}
+    cons_kwargs = {}  # type: Dict[str, Any]
+    call_kwargs = {}  # type: Dict[str, Any]
     f = None
     method = None
 
@@ -146,7 +144,7 @@ def _apply_over_vars_with_dim(func, self, dim=None, **kwargs):
 
     ds = type(self)(coords=self.coords, attrs=self.attrs)
 
-    for name, var in iteritems(self.data_vars):
+    for name, var in self.data_vars.items():
         if dim in var.dims:
             ds[name] = func(var, dim=dim, **kwargs)
         else:
@@ -370,7 +368,7 @@ def _get_valid_fill_mask(arr, dim, limit):
     None'''
     kw = {dim: limit + 1}
     # we explicitly use construct method to avoid copy.
-    new_dim = rolling._get_new_dimname(arr.dims, '_window')
+    new_dim = utils.get_temp_dimname(arr.dims, '_window')
     return (arr.isnull().rolling(min_periods=1, **kw)
             .construct(new_dim, fill_value=False)
             .sum(new_dim, skipna=False)) <= limit
@@ -413,10 +411,9 @@ def _floatize_x(x, new_x):
             # We assume that the most of the bits are used to represent the
             # offset (min(x)) and the variation (x - min(x)) can be
             # represented by float.
-            xmin = x[i].min()
-            x[i] = datetime_to_numeric(x[i], offset=xmin, dtype=np.float64)
-            new_x[i] = datetime_to_numeric(
-                new_x[i], offset=xmin, dtype=np.float64)
+            xmin = x[i].values.min()
+            x[i] = x[i]._to_numeric(offset=xmin, dtype=np.float64)
+            new_x[i] = new_x[i]._to_numeric(offset=xmin, dtype=np.float64)
     return x, new_x
 
 
