@@ -1,20 +1,18 @@
 """Internal utilties; not for external use
 """
-from __future__ import absolute_import, division, print_function
-
 import contextlib
 import functools
 import itertools
 import os.path
 import re
 import warnings
-from collections import Iterable, Mapping, MutableMapping, MutableSet
+from collections import OrderedDict
+from collections.abc import Iterable, Mapping, MutableMapping, MutableSet
 
 import numpy as np
 import pandas as pd
 
-from .pycompat import (
-    OrderedDict, basestring, bytes_type, dask_array_type, iteritems)
+from .pycompat import dask_array_type
 
 
 def _check_inplace(inplace, default=False):
@@ -22,7 +20,8 @@ def _check_inplace(inplace, default=False):
         inplace = default
     else:
         warnings.warn('The inplace argument has been deprecated and will be '
-                      'removed in xarray 0.12.0.', FutureWarning, stacklevel=3)
+                      'removed in a future version of xarray.',
+                      FutureWarning, stacklevel=3)
 
     return inplace
 
@@ -37,7 +36,7 @@ def alias_warning(old_name, new_name, stacklevel=3):
 
 
 def alias(obj, old_name):
-    assert isinstance(old_name, basestring)
+    assert isinstance(old_name, str)
 
     @functools.wraps(obj)
     def wrapper(*args, **kwargs):
@@ -157,7 +156,7 @@ def update_safety_check(first_dict, second_dict, compat=equivalent):
         Binary operator to determine if two values are compatible. By default,
         checks for equivalence.
     """
-    for k, v in iteritems(second_dict):
+    for k, v in second_dict.items():
         if k in first_dict and not compat(v, first_dict[k]):
             raise ValueError('unsafe to merge dictionaries without '
                              'overriding values; conflicting key %r' % k)
@@ -212,7 +211,7 @@ def is_scalar(value):
     """
     return (
         getattr(value, 'ndim', None) == 0 or
-        isinstance(value, (basestring, bytes_type)) or not
+        isinstance(value, (str, bytes)) or not
         isinstance(value, (Iterable, ) + dask_array_type))
 
 
@@ -488,7 +487,7 @@ class NDArrayMixin(NdimSizeLenMixin):
 class ReprObject(object):
     """Object that prints as the given value, for use with sentinel values."""
 
-    def __init__(self, value):  # type: str
+    def __init__(self, value: str):
         self._value = value
 
     def __repr__(self):
@@ -552,7 +551,7 @@ def decode_numpy_dict_values(attrs):
         if isinstance(v, np.ndarray):
             attrs[k] = v.tolist()
         elif isinstance(v, np.generic):
-            attrs[k] = np.asscalar(v)
+            attrs[k] = v.item()
     return attrs
 
 
@@ -605,27 +604,19 @@ class HiddenKeyDict(MutableMapping):
         return len(self._data) - num_hidden
 
 
-def datetime_to_numeric(array, offset=None, datetime_unit=None, dtype=float):
-    """Convert an array containing datetime-like data to an array of floats.
+def get_temp_dimname(dims, new_dim):
+    """ Get an new dimension name based on new_dim, that is not used in dims.
+    If the same name exists, we add an underscore(s) in the head.
 
-    Parameters
-    ----------
-    da : array
-        Input data
-    offset: Scalar with the same type of array or None
-        If None, subtract minimum values to reduce round off error
-    datetime_unit: None or any of {'Y', 'M', 'W', 'D', 'h', 'm', 's', 'ms',
-        'us', 'ns', 'ps', 'fs', 'as'}
-    dtype: target dtype
-
-    Returns
-    -------
-    array
+    Example1:
+        dims: ['a', 'b', 'c']
+        new_dim: ['_rolling']
+        -> ['_rolling']
+    Example2:
+        dims: ['a', 'b', 'c', '_rolling']
+        new_dim: ['_rolling']
+        -> ['__rolling']
     """
-    if offset is None:
-        offset = array.min()
-    array = array - offset
-
-    if datetime_unit:
-        return (array / np.timedelta64(1, datetime_unit)).astype(dtype)
-    return array.astype(dtype)
+    while new_dim in dims:
+        new_dim = '_' + new_dim
+    return new_dim
