@@ -1,6 +1,3 @@
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
 from .plot import (plot, line, step, contourf, contour,
                    hist, imshow, pcolormesh)
 
