@@ -1,5 +1,3 @@
-from __future__ import absolute_import
-
 from datetime import timedelta
 
 import numpy as np
@@ -10,10 +8,11 @@ import xarray as xr
 from xarray.coding.cftimeindex import (
     CFTimeIndex, _parse_array_of_cftime_strings, _parse_iso8601_with_reso,
     _parsed_string_to_bounds, assert_all_valid_date_type, parse_iso8601)
-from xarray.tests import assert_array_equal, assert_identical
+from xarray.tests import assert_array_equal, assert_allclose, assert_identical
 
-from . import (has_cftime, has_cftime_1_0_2_1, has_cftime_or_netCDF4,
-               raises_regex, requires_cftime)
+from . import (
+    has_cftime, has_cftime_1_0_2_1, has_cftime_or_netCDF4, raises_regex,
+    requires_cftime)
 from .test_coding_times import (
     _ALL_CALENDARS, _NON_STANDARD_CALENDARS, _all_cftime_date_types)
 
@@ -376,15 +375,9 @@ def test_groupby(da):
     assert_identical(result, expected)
 
 
-@pytest.mark.skipif(not has_cftime, reason='cftime not installed')
-def test_resample_error(da):
-    with pytest.raises(NotImplementedError, match='to_datetimeindex'):
-        da.resample(time='Y')
-
-
 SEL_STRING_OR_LIST_TESTS = {
     'string': '0001',
-    'string-slice': slice('0001-01-01', '0001-12-30'),
+    'string-slice': slice('0001-01-01', '0001-12-30'),  # type: ignore
     'bool-list': [True, True, False, False]
 }
 
@@ -586,6 +579,14 @@ def test_indexing_in_series_iloc(series, index):
     assert series.iloc[:2].equals(expected)
 
 
+@pytest.mark.skipif(not has_cftime, reason='cftime not installed')
+def test_series_dropna(index):
+    series = pd.Series([0., 1., np.nan, np.nan], index=index)
+    expected = series.iloc[:2]
+    result = series.dropna()
+    assert result.equals(expected)
+
+
 @pytest.mark.skipif(not has_cftime, reason='cftime not installed')
 def test_indexing_in_dataframe_loc(df, index, scalar_args, range_args):
     expected = pd.Series([1], name=index[0])
@@ -701,6 +702,26 @@ def test_cftimeindex_sub_cftimeindex(calendar):
     assert isinstance(result, pd.TimedeltaIndex)
 
 
+@pytest.mark.skipif(not has_cftime, reason='cftime not installed')
+@pytest.mark.parametrize('calendar', _CFTIME_CALENDARS)
+def test_cftimeindex_sub_cftime_datetime(calendar):
+    a = xr.cftime_range('2000', periods=5, calendar=calendar)
+    result = a - a[0]
+    expected = pd.TimedeltaIndex([timedelta(days=i) for i in range(5)])
+    assert result.equals(expected)
+    assert isinstance(result, pd.TimedeltaIndex)
+
+
+@pytest.mark.skipif(not has_cftime, reason='cftime not installed')
+@pytest.mark.parametrize('calendar', _CFTIME_CALENDARS)
+def test_cftime_datetime_sub_cftimeindex(calendar):
+    a = xr.cftime_range('2000', periods=5, calendar=calendar)
+    result = a[0] - a
+    expected = pd.TimedeltaIndex([timedelta(days=-i) for i in range(5)])
+    assert result.equals(expected)
+    assert isinstance(result, pd.TimedeltaIndex)
+
+
 @pytest.mark.skipif(not has_cftime, reason='cftime not installed')
 @pytest.mark.parametrize('calendar', _CFTIME_CALENDARS)
 def test_cftimeindex_sub_timedeltaindex(calendar):
