@@ -1,5 +1,4 @@
-from __future__ import absolute_import, division, print_function
-
+from collections import OrderedDict
 from copy import deepcopy
 from itertools import product
 
@@ -12,10 +11,9 @@ from xarray.core.combine import (
     _auto_combine, _auto_combine_1d, _auto_combine_all_along_first_dim,
     _check_shape_tile_ids, _combine_nd, _infer_concat_order_from_positions,
     _infer_tile_ids_from_nested_list, _new_tile_id)
-from xarray.core.pycompat import OrderedDict, iteritems
 
 from . import (
-    InaccessibleArray, assert_array_equal, assert_combined_tile_ids_equal,
+    InaccessibleArray, assert_array_equal,
     assert_equal, assert_identical, raises_regex, requires_dask)
 from .test_dataset import create_test_data
 
@@ -38,7 +36,7 @@ class TestConcatDataset(object):
             # return a new dataset with all variable dimensions transposed into
             # the order in which they are found in `data`
             return Dataset(dict((k, v.transpose(*data[k].dims))
-                                for k, v in iteritems(dataset.data_vars)),
+                                for k, v in dataset.data_vars.items()),
                            dataset.coords, attrs=dataset.attrs)
 
         for dim in ['dim1', 'dim2']:
@@ -52,7 +50,7 @@ class TestConcatDataset(object):
             data, concat(datasets, data[dim], coords='minimal'))
 
         datasets = [g for _, g in data.groupby(dim, squeeze=True)]
-        concat_over = [k for k, v in iteritems(data.coords)
+        concat_over = [k for k, v in data.coords.items()
                        if dim in v.dims and k != dim]
         actual = concat(datasets, data[dim], coords=concat_over)
         assert_identical(data, rectify_dim_order(actual))
@@ -287,6 +285,15 @@ class TestConcatDataArray(object):
         assert concat([foo, foo], dim="x").encoding == foo.encoding
         assert concat([ds, ds], dim="x").encoding == ds.encoding
 
+    @pytest.mark.parametrize("colors, expected_name",
+                             [(['blue', 'green', 'red'], None),
+                              (['red', 'red', 'red'], 'red')])
+    def test_concat_determine_name(self, colors, expected_name):
+        das = [DataArray(np.random.random((2, 2)), dims=['x', 'y'], name=k)
+               for k in colors]
+        result = concat(das, dim="band")
+        assert result.name is expected_name
+
     @requires_dask
     def test_concat_lazy(self):
         import dask.array as da
@@ -411,6 +418,13 @@ class TestAutoCombine(object):
         assert_identical(expected, actual)
 
 
+def assert_combined_tile_ids_equal(dict1, dict2):
+    assert len(dict1) == len(dict2)
+    for k, v in dict1.items():
+        assert k in dict2.keys()
+        assert_equal(dict1[k], dict2[k])
+
+
 class TestTileIDsFromNestedList(object):
     def test_1d(self):
         ds = create_test_data
