@@ -1,5 +1,3 @@
-from __future__ import absolute_import, division, print_function
-
 import numpy as np
 import pandas as pd
 import pytest
@@ -293,7 +291,7 @@ def test_errors(use_dask):
     if use_dask:
         da = get_example_data(3)
     else:
-        da = get_example_data(1)
+        da = get_example_data(0)
 
     result = da.interp(x=[-1, 1, 3], kwargs={'fill_value': 0.0})
     assert not np.isnan(result.values).any()
@@ -525,7 +523,8 @@ def test_cftime_type_error():
 def test_cftime_list_of_strings():
     from cftime import DatetimeProlepticGregorian
 
-    times = xr.cftime_range('2000', periods=24, freq='D')
+    times = xr.cftime_range('2000', periods=24, freq='D',
+                            calendar='proleptic_gregorian')
     da = xr.DataArray(np.arange(24), coords=[times], dims='time')
 
     times_new = ['2000-01-01T12:00', '2000-01-02T12:00', '2000-01-03T12:00']
@@ -544,7 +543,8 @@ def test_cftime_list_of_strings():
 def test_cftime_single_string():
     from cftime import DatetimeProlepticGregorian
 
-    times = xr.cftime_range('2000', periods=24, freq='D')
+    times = xr.cftime_range('2000', periods=24, freq='D',
+                            calendar='proleptic_gregorian')
     da = xr.DataArray(np.arange(24), coords=[times], dims='time')
 
     times_new = '2000-01-01T12:00'
@@ -573,3 +573,16 @@ def test_cftime_to_non_cftime_error():
 
     with pytest.raises(TypeError):
         da.interp(time=0.5)
+
+
+@requires_scipy
+def test_datetime_interp_noerror():
+    # GH:2667
+    a = xr.DataArray(
+        np.arange(21).reshape(3, 7), dims=['x', 'time'],
+        coords={'x': [1, 2, 3],
+                'time': pd.date_range('01-01-2001', periods=7, freq='D')})
+    xi = xr.DataArray(
+        np.linspace(1, 3, 50), dims=['time'],
+        coords={'time': pd.date_range('01-01-2001', periods=50, freq='H')})
+    a.interp(x=xi, time=xi.time)  # should not raise an error
